angular.module("PMPMobile").factory('LoadingInterceptor', function($q, $rootScope) {

	$rootScope.pendingRequests = 0;

	function activeLoadingStatus() {
		if ($rootScope.pendingRequests !== 0) {
			$rootScope.$broadcast("activeLoadingStatus");
		}
	}

	function inActiveLoadingStatus() {
		if ($rootScope.pendingRequests === 0) {
			$rootScope.$broadcast("inActiveLoadingStatus");
		}
	}

	function addPendingRequest() {
		$rootScope.pendingRequests++;
		activeLoadingStatus();
	}

	function removePendingRequest() {
		$rootScope.pendingRequests--;
		inActiveLoadingStatus();
	}

	return {
		'request': function(config) {
			if (config.url.indexOf("http") == 0 || config.url.indexOf("https") == 0) {
				addPendingRequest();
			}
			return config || $q.when(config);
			
		},

		'requestError': function(rejection) {
			if (rejection.config.url.indexOf("http") == 0 || rejection.config.url.indexOf("https") == 0) {
				removePendingRequest();
			}
			return $q.reject(rejection);
		},

		'response': function(response) {
			if (response.config.url.indexOf("http") == 0 || response.config.url.indexOf("https") == 0) {
				removePendingRequest();
			}
			return response || $q.when(response);
		},

		'responseError': function(rejection) {
			if (rejection.config.url.indexOf("http") == 0 || rejection.config.url.indexOf("https") == 0) {
				removePendingRequest();
			}
			return $q.reject(rejection);
		}
	};
}).config(function($httpProvider) {
	$httpProvider.interceptors.push('LoadingInterceptor');
});